import platform

from localhttps.keychain.linux import LinuxKeychain
from localhttps.keychain.macos import MacOSKeychain


def get_current_keychain():
    name = platform.system()

    if name == 'Linux':
        return LinuxKeychain()
    elif name == 'Darwin':
        return MacOSKeychain()
    else:
        raise ValueError(f'Platform {name} is not supported')



from typing import AsyncIterable
from aiopath import AsyncPath
from aiofiles.os import wrap
import shutil
import rich

from localhttps.cert.ca import CertificationAuthority
from localhttps.keychain.abc import AbstractKeychain
from localhttps.cmd import Cmd
from localhttps.utils.cli import ask_to_run_command


which = wrap(shutil.which)

class LinuxKeychain(AbstractKeychain):
    async def databases(self) -> AsyncIterable[str]:
        p = (await AsyncPath.home())/'.pki'/'nssdb'
        if await p.exists():
            yield f'sql:{await p.absolute()}'

        async for p in ((await AsyncPath.home())/'.mozilla'/'firefox').glob('*.default'):
            yield str(await p.absolute())

        yield f'p11-kit'

    async def trust_ca(self, cmd: Cmd, ca: CertificationAuthority, database: str):
        console = rich.get_console()

        if database == 'p11-kit':
            trust_path = await which('trust')
            if trust_path is None:
                console.print()
                console.print('Install p11-kit and then:')

            await ask_to_run_command(f'sudo trust anchor --store {await ca.pem_path.absolute()}')
            return

        await cmd.run(
            'certutil',
            '-d', database,
            '-A',
            '-t', 'TC',
            '-n', 'localhttps',
            '-i', str(await ca.pem_path.absolute()),
        )



from typing import List
import asyncclick as click
import json
from localhttps.cli import with_context
from localhttps.cli.commands import cli
from localhttps.cli.context import Context
from localhttps.utils.domain import normalize_domain

DEFAULT_FORMATS = {
    'webpack': "--client-web-socket-url-hostname={domain} --server-type https --server-options-key={cert[key]} --server-options-cert={cert[crt]} --server-options-ca={ca[pem]}",
}

@cli.command(help='Create certificate and print it with given format')
@click.option('--ca', default='default', help='Certification authority name')
@click.option('--force/--no-force', default=False, help='Force recreate')
@click.argument('format')
@click.argument('domain', nargs=-1)
@with_context
async def use(ctx: Context, ca: str, force: bool, format: str, domain: List[str]):
    ca = ctx.app.ca(ca)
    domains = domain

    if not await ca.exists():
        ctx.console.print(f'[red]certification authority [blue]{ca.name}[/blue] is not created, use init command to create first[/red]')
        ctx.exit(1)

    is_first = True
    for domain in domains:
        domain = normalize_domain(domain)

        cert = ctx.app.cert(
            domain=domain,
            name=domain,
            ca=ca,
        )

        if force or not await cert.exists():
            ctx.console.print(f'creating certificate [blue]{domain}[/blue]...')
            await cert.create(ctx.cmd)
            ctx.console.print(f'certificate [blue]{domain}[/blue] created!')

        if format in DEFAULT_FORMATS:
            format = DEFAULT_FORMATS[format]

        if is_first:
            is_first = False
        else:
            print()

        if format == 'json':
            print(
                json.dumps({
                    'domain': cert.domain,

                    'cert': {
                        'key': str(await cert.key_path.absolute()),
                        'crt': str(await cert.crt_path.absolute()),
                    },
                    'ca': {
                        'key': str(await ca.key_path.absolute()),
                        'pem': str(await ca.pem_path.absolute()),
                    },
                }, separators=(',', ':')),
                end='',
            )
        else:
            print(
                format.format(
                    domain=cert.domain,

                    cert={
                        'key': str(await cert.key_path.absolute()),
                        'crt': str(await cert.crt_path.absolute()),
                    },

                    ca = {
                        'key': str(await ca.key_path.absolute()),
                        'pem': str(await ca.pem_path.absolute()),
                    },
                ),
                end='',
            )



from typing import NoReturn
import asyncclick as click
from aiopath import AsyncPath
from rich.console import Console

from localhttps.app import App
from localhttps.cmd import Cmd
from localhttps.keychain.abc import AbstractKeychain


class Context:
    click: click.core.Context
    cmd: Cmd
    console: Console
    data_path: AsyncPath
    app: App
    keychain: AbstractKeychain

    def exit(self, code: int = 0) -> NoReturn:
        self.click.exit(code)



import re

domain_regex = re.compile(r'^(?:https?://)?([^/]+)')

def normalize_domain(domain: str) -> str:
    match = domain_regex.match(domain)
    if match is not None:
        return match.group(1)
    else:
        return domain



from typing import AsyncIterable, List

from localhttps.cert.ca import CertificationAuthority
from localhttps.keychain.abc import AbstractKeychain
from localhttps.cmd import Cmd
from localhttps.utils.cli import ask_to_run_command


class MacOSKeychain(AbstractKeychain):
    async def databases(self) -> AsyncIterable[str]:
        yield '/Library/Keychains/System.keychain'

    async def trust_ca(self, cmd: Cmd, ca: CertificationAuthority, database: str):
        await ask_to_run_command(f'sudo security add-trusted-cert -d -r trustRoot -k /Library/Keychains/System.keychain "{await ca.pem_path.absolute()}"')



import rich

async def ask_to_run_command(command: str) -> None:
    console = rich.get_console()
    console.print('')
    console.print('Execute the following command in terminal and press enter:')
    console.print('')
    console.print(f'    {command}')
    console.print('')
    console.input('[Press enter]')



from typing import Optional
import asyncclick as click
import os
from aiopath import AsyncPath
from localhttps.app import App
from localhttps.cli import with_context, Context
from localhttps.cmd import DefaultConsoleCmd


async def resolve_default_data_path() -> AsyncPath:
    data_path = os.environ.get('LOCALHTTPS_DATA', default=None)

    if data_path is not None:
        data_path = await AsyncPath(data_path).absolute()
    else:
        config_home = os.environ.get('XDG_CONFIG_HOME', default=None)
        if config_home is not None:
            config_home = await AsyncPath(config_home).absolute()
        else:
            config_home = await ((await AsyncPath.home())/'.config').absolute()

        data_path: AsyncPath = config_home/'localhttps'

    await data_path.mkdir(exist_ok=True)
    return data_path

@click.group()
@click.option('--data-path', default=None, help='Base data path (defaults to $LOCALHTTPS_DATA or $XDG_CONFIG_HOME/localhttps (~/.config/localhttps))')
@click.option('--verbose/-V', default=False, help='Verbose output')
@with_context
async def cli(ctx: Context, data_path: Optional[str], verbose: bool):
    if data_path is not None:
        ctx.data_path = await AsyncPath(data_path).absolute()
    else:
        ctx.data_path = await resolve_default_data_path()

    ctx.app = App(ctx.data_path)

    if verbose and isinstance(ctx.cmd, DefaultConsoleCmd):
        ctx.cmd.verbose = True

import localhttps.cli.commands.init
import localhttps.cli.commands.secure
import localhttps.cli.commands.status
import localhttps.cli.commands.unsecure
import localhttps.cli.commands.use



from abc import ABC, abstractmethod
from typing import AsyncIterable
from localhttps.cert.ca import CertificationAuthority

from localhttps.cmd import Cmd


class AbstractKeychain(ABC):
    @abstractmethod
    async def databases(self) -> AsyncIterable[str]:
        pass

    @abstractmethod
    async def trust_ca(self, cmd: Cmd, ca: CertificationAuthority, database: str):
        pass



import asyncio
from typing import List, Tuple
from aiopath import AsyncPath

from localhttps.cmd import Cmd

class CertificationAuthority:
    _root_path: AsyncPath
    _name: str

    def __init__(self, root_path: AsyncPath, name: str) -> None:
        self._root_path = root_path
        self._name = name

    @property
    def path(self) -> AsyncPath:
        return self._root_path

    @property
    def name(self) -> str:
        return self._name

    @property
    def key_path(self) -> AsyncPath:
        return self._root_path/f'{self._name}.key'

    @property
    def pem_path(self) -> AsyncPath:
        return self._root_path/f'{self._name}.pem'

    @property
    def srl_path(self) -> AsyncPath:
        return self._root_path/f'{self._name}.srl'

    @property
    def org_name(self) -> str:
        return 'LocalHTTPS CA Self Signed Organization'

    @property
    def common_name(self) -> str:
        return 'LocalHTTPS CA Self Signed CN'

    @property
    def email(self) -> str:
        return f'{self._name}@localhttps'

    @property
    def subject_parts(self) -> List[Tuple[str, str]]:
        return [
            ('C', ''),
            ('ST', ''),
            ('O', self.org_name),
            ('localityName', ''),
            ('commonName', self.common_name),
            ('organizationalUnitName', 'Developers'),
            ('emailAddress', self.email),
        ]

    @property
    def subject(self) -> str:
        return ''.join(
            f'/{key}={value}' for (key, value) in self.subject_parts
        ) + '/'

    async def exists(self) -> bool:
        key_exists, pem_exists = await asyncio.gather(self.key_path.exists(), self.pem_path.exists())
        return key_exists and pem_exists

    async def create(self, cmd: Cmd) -> None:
        await self.delete()
        await self._root_path.mkdir(parents=True, exist_ok=True)

        await cmd.run(
            'openssl',
            'req',
            '-new',
            '-newkey', 'rsa:2048',
            '-days', '730',
            '-nodes',
            '-x509',
            '-subj',
            self.subject,
            '-keyout', str(await self.key_path.absolute()),
            '-out', str(await self.pem_path.absolute()),
        )

    async def delete(self) -> None:
        await asyncio.gather(
            self.key_path.unlink(missing_ok=True),
            self.pem_path.unlink(missing_ok=True),
        )
        if await self._root_path.exists():
            async for _ in self._root_path.glob('*'):
                return
            self._root_path.rmdir()



import asyncclick as click
from localhttps.cli import with_context
from localhttps.cli.commands import cli
from localhttps.cli.context import Context


@cli.command(help='Initialize certification authority')
@click.option('--force-ca/--no-force-ca', default=False, help='Force create certification authority')
@click.option('--trust/--no-trust', default=False, help='Add certification authority to system keychain/browser keychains')
@click.option('--nginx/--no-nginx', default=False, help='Generate config for nginx')
@click.option('--ca', default='default', help='Certification authority name')
@with_context
async def init(ctx: Context, force_ca: bool, trust: bool, nginx: bool, ca: str):
    ca = ctx.app.ca(ca)

    if not force_ca:
        ctx.console.print('checking...')
    if force_ca or not await ca.exists():
        ctx.console.print('creating...')
        await ca.create(ctx.cmd)
        ctx.console.print('created!')
    else:
        ctx.console.print('already exists')

    ctx.console.print(f'located in {await ca.path.absolute()} with name {ca.name}')

    if trust:
        async for db in ctx.keychain.databases():
            ctx.console.print(f'adding to [blue]{db}[/blue]')
            await ctx.keychain.trust_ca(ctx.cmd, ca, database=db)

    if nginx:
        out_path = await (ctx.data_path/'Nginx'/'ssl'/f'{ca.name}.conf').absolute()
        await out_path.parent.mkdir(parents=True, exist_ok=True)
        await ctx.app.generate_universal_nginx_config(ca, out_path)
        ctx.console.print(f'nginx config generated to {out_path}')



from abc import ABC, abstractmethod
import asyncio
from typing import Tuple
from rich.console import Console
from rich.text import Text


class Cmd(ABC):
    @abstractmethod
    async def run(self, command, *args, **kwargs) -> asyncio.subprocess.Process:
        pass


class DefaultConsoleCmd(Cmd):
    _console: Console

    verbose = False

    def __init__(self, console: Console) -> None:
        self._console = console

    async def _capture_output(self, stream: asyncio.StreamReader, label: str) -> bytes:
        buffer = b''
        while True:
            line = await stream.readline()
            if line == b'':
                break
            buffer += line

            if self.verbose:
                if not line.endswith(b'\n'):
                    self._console.print(label, Text(line.decode('utf-8')), '%', sep='')
                else:
                    self._console.print(label, Text(line.decode('utf-8')[0:-1]), sep='')

        return buffer

    async def run(self, command, *args, **kwargs) -> Tuple[int, bytes, bytes]:
        full_command = f'{command} {" ".join(args)}'
        if self.verbose:
            self._console.print(f'$ [blue]{full_command}[/blue]')
        with self._console.status(command):
            proc = await asyncio.subprocess.create_subprocess_exec(
                command,
                *args,
                **kwargs,
                stdout=asyncio.subprocess.PIPE,
                stderr=asyncio.subprocess.PIPE,
            )

            stdout, stderr, exit_code = await asyncio.gather(
                self._capture_output(proc.stdout, label='[blue]\[stdout][/blue] '),
                self._capture_output(proc.stderr, label='[blue]\[stdout][/blue] '),
                proc.wait()
            )

            if self.verbose or exit_code != 0:
                self._console.print(f'[blue]{command}[/blue] exited with code [blue]{exit_code}[/blue]')



from typing import List
import asyncclick as click
from localhttps.cli import with_context
from localhttps.cli.commands import cli
from localhttps.cli.context import Context
from localhttps.utils.domain import normalize_domain


@cli.command(help='Create certificate for domain')
@click.option('--ca', default='default', help='Certification authority name')
@click.option('--force/--no-force', default=False, help='Force recreate')
@click.option('--nginx/--no-nginx', default=False, help='Generate config for nginx')
@click.argument('domain', nargs=-1)
@with_context
async def secure(ctx: Context, ca: str, force: bool, nginx: bool, domain: List[str]):
    ca = ctx.app.ca(ca)
    domains = domain

    if not await ca.exists():
        ctx.console.print(f'[red]certification authority [blue]{ca.name}[/blue] is not created, use init command to create first[/red]')
        ctx.exit(1)

    for domain in domains:
        domain = normalize_domain(domain)

        cert = ctx.app.cert(
            domain=domain,
            name=domain,
            ca=ca,
        )

        if force or not await cert.exists():
            ctx.console.print(f'creating certificate [blue]{domain}[/blue]...')
            await cert.create(ctx.cmd)
            ctx.console.print('certificate created!')
        else:
            ctx.console.print('certificate already exists, use [blue]--force[/blue] to recreate')

        if nginx:
            out_path = await (ctx.data_path/'Nginx'/'ssl'/f'{domain}.conf').absolute()
            await out_path.parent.mkdir(parents=True, exist_ok=True)
            await ctx.app.generate_nginx_config(cert, out_path)
            ctx.console.print(f'nginx config generated to {out_path}')



from typing import List
import asyncclick as click
from localhttps.cli import with_context
from localhttps.cli.commands import cli
from localhttps.cli.context import Context
from localhttps.utils.domain import normalize_domain


@cli.command(help='Delete certificate for domain')
@click.option('--ca', default='default', help='Certification authority name')
@click.argument('domain', nargs=-1)
@with_context
async def unsecure(ctx: Context, ca: str, domain: List[str]):
    ca = ctx.app.ca(ca)
    domains = domain

    if not await ca.exists():
        ctx.console.print(f'[red]certification authority [blue]{ca.name}[/blue] is not created[/red]')
        ctx.exit(1)

    for domain in domains:
        domain = normalize_domain(domain)

        cert = ctx.app.cert(
            domain=domain,
            name=domain,
            ca=ca,
        )

        if not await cert.exists():
            ctx.console.print(f'[red]certificate [blue]{domain}[/blue] not found[/red]')
            ctx.exit(1)

        await cert.delete()

        ctx.console.print(f'certificate [blue]{domain}[/blue] removed')

        nginx_path = await (ctx.data_path/'Nginx'/'ssl'/f'{domain}.conf').absolute()
        if await nginx_path.exists():
            await nginx_path.unlink(missing_ok=True)
            ctx.console.print(f'deleted [blue]{nginx_path}[/blue]')



import functools
import asyncclick as click
from localhttps.cli.context import Context

def with_context(func):
    @click.pass_context
    @functools.wraps(func)
    def wrapper(ctx: click.core.Context, *args, **kwargs):
        ctx.obj.click = ctx
        return func(ctx.obj, *args, **kwargs)
    return wrapper



from typing import List
import asyncclick as click
from localhttps.cert.ca import CertificationAuthority
from localhttps.cli import with_context
from localhttps.cli.commands import cli
from localhttps.cli.context import Context


async def desync(it):
    for x in it: yield x

@cli.command(help='List CA and their certificates')
@click.argument('ca', nargs=-1)
@with_context
async def status(ctx: Context, ca: List[str]):
    if len(ca) == 0:
        authorities = ctx.app.list_ca()
    else:
        authorities = desync(ctx.app.ca(name) for name in ca)

    async for ca in authorities:
        ca: CertificationAuthority

        if await ca.exists():
            ctx.console.print(f'ca [blue]{ca.name}[/blue]:')

            async for cert in ctx.app.list_certs(ca):
                ctx.console.print(f' - [blue]{cert.name}[/blue] (https://{cert.domain})')
        else:
            ctx.console.print(f'[red]ca [blue]{ca.name}[/blue]: not created[/red]')



import asyncio
from typing import List, Tuple
from aiopath import AsyncPath

from localhttps.cert.ca import CertificationAuthority
from localhttps.cmd import Cmd

class Certificate:
    _authority: CertificationAuthority
    _root_path: AsyncPath
    _name: str
    _domain: str

    def __init__(self, authority: CertificationAuthority, root_path: AsyncPath, name: str, domain: str) -> None:
        self._authority = authority
        self._root_path = root_path
        self._name = name
        self._domain = domain

    @property
    def authority(self) -> CertificationAuthority:
        return self._authority

    @property
    def name(self) -> str:
        return self._name

    @property
    def domain(self) -> str:
        return self._domain

    @property
    def key_path(self) -> AsyncPath:
        return self._root_path/f'{self._name}.key'

    @property
    def csr_path(self) -> AsyncPath:
        return self._root_path/f'{self._name}.csr'

    @property
    def crt_path(self) -> AsyncPath:
        return self._root_path/f'{self._name}.crt'

    @property
    def conf_path(self) -> AsyncPath:
        return self._root_path/f'{self._name}.conf'

    @property
    def common_name(self) -> str:
        return self._domain

    @property
    def email(self) -> str:
        return f'{self._domain}@localhttps'

    @property
    def subject_parts(self) -> List[Tuple[str, str]]:
        return [
            ('C', ''),
            ('ST', ''),
            ('O', ''),
            ('localityName', ''),
            ('commonName', self.common_name),
            ('organizationalUnitName', ''),
            ('emailAddress', self.email),
        ]

    @property
    def subject(self) -> str:
        return ''.join(
            f'/{key}={value}' for (key, value) in self.subject_parts
        ) + '/'

    async def exists(self) -> bool:
        key_exists, csr_exists, crt_exists, conf_exists = await asyncio.gather(
            self.key_path.exists(),
            self.csr_path.exists(),
            self.crt_path.exists(),
            self.conf_path.exists(),
        )
        return key_exists and csr_exists and crt_exists and conf_exists

    async def _build_certificate_conf(self) -> None:
        await self.conf_path.write_text(f'''
# Generated by LocalHTTPS
# domain={self._domain}

[req]
distinguished_name = req_distinguished_name
req_extensions = v3_req

[req_distinguished_name]
countryName = Country Name (2 letter code)
countryName_default = US
stateOrProvinceName = State or Province Name (full name)
stateOrProvinceName_default = MN
localityName = Locality Name (eg, city)
localityName_default = Minneapolis
organizationalUnitName	= Organizational Unit Name (eg, section)
organizationalUnitName_default	= Domain Control Validated
commonName = Internet Widgits Ltd
commonName_max	= 64

[ v3_req ]
# Extensions to add to a certificate request
basicConstraints = CA:FALSE
keyUsage = nonRepudiation, digitalSignature, keyEncipherment
subjectAltName = @alt_names

[alt_names]
DNS.1 = {self._domain}
DNS.2 = *.{self._domain}
'''.strip())

    async def _create_private_key(self, cmd: Cmd) -> None:
        await cmd.run(
            'openssl',
            'genrsa',
            '-out', str(await self.key_path.absolute()),
            '2048',
        )

    async def _create_signing_request(self, cmd: Cmd) -> None:
        await cmd.run(
            'openssl',
            'req',
            '-new',
            '-key', str(await self.key_path.absolute()),
            '-out', str(await self.csr_path.absolute()),
            '-subj', self.subject,
            '-config', str(await self.conf_path.absolute()),
        )

    async def create(self, cmd: Cmd) -> None:
        await self.delete()
        await self._root_path.mkdir(parents=True, exist_ok=True)

        await asyncio.gather(
            self._build_certificate_conf(),
            self._create_private_key(cmd),
        )
        await self._create_signing_request(cmd)

        params = [
            '-req',
            '-sha256',
            '-days', '730',
            '-CA', str(await self._authority.pem_path.absolute()),
            '-CAkey', str(await self._authority.key_path.absolute()),
            '-CAserial', str(await self._authority.srl_path.absolute()),
        ]

        if not await self._authority.srl_path.exists():
            params.append('-CAcreateserial')

        params = [
            *params,
            '-in', str(await self.csr_path.absolute()),
            '-out', str(await self.crt_path.absolute()),
            '-extensions', 'v3_req',
            '-extfile', str(await self.conf_path.absolute()),
        ]

        await cmd.run('openssl', 'x509', *params)
        await self.crt_path.chmod(0o644)
        await self.key_path.chmod(0o644)

    async def delete(self) -> None:
        await asyncio.gather(
            self.key_path.unlink(missing_ok=True),
            self.csr_path.unlink(missing_ok=True),
            self.crt_path.unlink(missing_ok=True),
            self.conf_path.unlink(missing_ok=True),
        )
        if await self._root_path.exists():
            async for _ in self._root_path.glob('*'):
                return
            await self._root_path.rmdir()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['localhttps',
 'localhttps.cert',
 'localhttps.cli',
 'localhttps.cli.commands',
 'localhttps.keychain',
 'localhttps.utils']

package_data = \
{'': ['*']}

install_requires = \
['aiofiles>=0.8.0,<0.9.0',
 'aiopath>=0.6.10,<0.7.0',
 'anyio>=3.5.0,<4.0.0',
 'asyncclick>=8.0.3,<9.0.0',
 'rich>=11.2.0,<12.0.0']

entry_points = \
{'console_scripts': ['localhttps = localhttps:main']}

setup_kwargs = {
    'name': 'localhttps',
    'version': '0.4.1',
    'description': 'HTTPS manager for local sites',
    'long_description': None,
    'author': 'Tarik02',
    'author_email': 'Taras.Fomin@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



import aiofiles
import os
import re

from typing import AsyncIterable, Optional
from aiopath import AsyncPath

from localhttps.cert.ca import CertificationAuthority
from localhttps.cert.cert import Certificate

class App:
    _data_path: AsyncPath

    def __init__(self, data_path: AsyncPath) -> None:
        self._data_path = data_path

    async def list_ca(self) -> AsyncIterable[CertificationAuthority]:
        root = self._data_path/'CertificationAuthorities'

        async for file in root.glob('*.pem'):
            yield CertificationAuthority(root, os.path.splitext(file.name)[0])

    def ca(self, name: str = 'default') -> CertificationAuthority:
        return CertificationAuthority(self._data_path/'CertificationAuthorities', name)

    async def list_certs(self, ca: Optional[CertificationAuthority] = None) -> AsyncIterable[Certificate]:
        if ca is None:
            ca = self.ca()

        root = self._data_path/'Certificates'/ca.name

        async for file in root.glob('*.conf'):
            name = os.path.splitext(file.name)[0]
            domain = name

            async with aiofiles.open(file) as f:
                if await f.readline() == '# Generated by LocalHTTPS\n':
                    match = re.match('^# domain=(.*)\n$', await f.readline())
                    if match is not None:
                        domain = match.group(1)

            yield Certificate(
                authority=ca,
                root_path=root,
                name=name,
                domain=domain
            )

    def cert(self, name: str, domain: str, ca: Optional[CertificationAuthority] = None) -> Certificate:
        if ca is None:
            ca = self.ca()

        return Certificate(
            authority=ca,
            root_path=self._data_path/'Certificates'/ca.name,
            name=name,
            domain=domain,
        )

    async def generate_nginx_config(self, cert: Certificate, out_path: AsyncPath):
        await out_path.write_text(f'''
ssl_certificate {await cert.crt_path.absolute()};
ssl_certificate_key {await cert.key_path.absolute()};
'''.strip())

    async def generate_universal_nginx_config(self, ca: CertificationAuthority, out_path: AsyncPath):
        key_path_prefix = await (self._data_path/'Certificates'/ca.name).absolute()
        await out_path.write_text(f'''
ssl_certificate {key_path_prefix}/$ssl_server_name.crt;
ssl_certificate_key {key_path_prefix}/$ssl_server_name.key;
'''.strip())



import asyncio
import asyncclick as click
from rich.console import Console
from localhttps.app import App
from localhttps.cli.commands import cli
from localhttps.cli.context import Context
from localhttps.cmd import DefaultConsoleCmd
from localhttps.keychain import get_current_keychain

async def create_context():
    ctx = Context()
    ctx.console = Console(soft_wrap=True, stderr=True)
    ctx.cmd = DefaultConsoleCmd(ctx.console)
    ctx.keychain = get_current_keychain()
    return ctx

def main():
    ctx = asyncio.run(create_context())
    cli(_anyio_backend='asyncio', obj=ctx)
